%
% File: invpend.m
% Inverted pendulum simulation with visualization
% and mouse input.
% Created by Marc Bodson. Last revised: 01/18/03.
% Revised by Taylor Meacham and Ronnie Boutte Last revised: 02/09/08
% Revised by Arn Stolp Last revised: 4/2/13
%

lbeam=1;               % beam total length (m) ADJUST LENGTH HERE
lbeam = input('Enter the beam length in meters (suggest between 2 and 6)');
cheat=1;    %    <= ADJUST Gravity CHEAT HERE (should stay at cheat=1)

g=9.81/cheat;          % acceleration of gravity (m/s^2)
lcg=lbeam/2;           % beam length to center of gravity (m)

% Set limits for simulation
thmax=pi/6;xmax=lbeam; % max. beam angle (rad) and cart position (m)
theta=0;xcart=0;       % initial beam angle (rad) and cart position (m)
%theta=5*pi/180;        % use to set an initial beam angle of 5 deg

% define model
dt=0.05;               % sampling period (s) -> 20Hz
a2=3*g/(4*lcg);nump=[-a2/g 0 0];denp=[1 0 -a2];sysp=tf(nump,denp);
[ap,bp,cp,dp]=ssdata(c2d(sysp,dt,'zoh'));f=5;
icca=inv([cp;cp*ap]);iccb=icca*([dp;cp*bp+dp]); % zero velocity initialization
xp=icca*[theta;theta]-iccb*xcart;xss=icca*[thmax;thmax];
% 
% Initialization - Controller
%
mode = input('Type 1 for manual control, 2 for automatic control:\n');
switch mode;
   case 1;gmanual=xmax;    % gain of manual controller
   case 2;invpendcinit;    % initialization of automatic controller
end;
%
% Initialization - Visualization
% Adjust the position & size of the simulation window below 
% to fit the screen and to square the axes (if needed)
% Format: [position from left, position from bottom, width, height]
%
figure(1);
wbeam=0.04;vgap=0.005*lbeam;wtrack=2.12*lbeam;
htrack=0.03*lbeam; %wcart=0.12*lbeam;hcart=0.05*lbeam;
wcart=0.24;hcart=0.1;
axis([-0.75*wtrack 0.75*wtrack 0 htrack+hcart+1.2*lbeam]);clf
axis([-0.75*wtrack 0.75*wtrack 0 htrack+hcart+1.2*lbeam]);
set(1,'pos',[20 300 970 393]);hold on % position/size of sim. window
xdtrack=[wtrack/2;wtrack/2;-wtrack/2;-wtrack/2];
ydtrack=[0;htrack;htrack;0];
ptrack=fill(xdtrack,ydtrack,'green'); 
xline1=[-lcg,-lcg];yline1=[0,htrack];
pline1=plot(xline1,yline1,'black','EraseMode','background');
xline2=[lcg,lcg];yline2=[0,htrack];
pline2=plot(xline2,yline2,'black','EraseMode','background');
xdcart=[wcart/2;wcart/2;-wcart/2;-wcart/2];
ydcart=[0;hcart;hcart;0];
pcart=fill(xdcart,ydcart+htrack+vgap,'blue','EraseMode','background'); 
xdbeam=[wbeam/2;wbeam/2;-wbeam/2;-wbeam/2];
ydbeam=[0;lbeam;lbeam;0];
pbeam=fill(xdbeam,ydbeam+htrack+vgap+vgap+hcart,'red','EraseMode','background'); 
%
% Real-time simulation
%
t=0;tm=0;nt=0;done=0;tic;
while (done==0);
   while tm<nt;tm=toc;end;t=nt;nt=t+dt;
%
% Controller
% Control signal:   xref, reference cart position (m)
% Useful variables: xcart, cart position (m)
%                   theta, beam angle (rad)
%                   t, time (s)
%
switch mode;
   case 1; %manual control
       monitor = get(0,'MonitorPosition');  % get monitor size done above
       cursor = get(0,'PointerLocation');   % get mouse pointer position on monitor
       sensitivity=15/lbeam;                % set sensitivity to mouse movement
       position=sensitivity*((cursor(1)-1)/(monitor(1,3)-1)-0.5);  % scale
       xcom=gmanual*position;

   case 2; % automatic control
       invpendc; 
end;
%
% Inverted pendulum model
%
xcart=xcart+dt*f*(xcom-xcart);
%xcart=xcom;
if (xcart>xmax);xcart=xmax;end;
if (xcart<-xmax);xcart=-xmax;end;
theta=cp*xp+dp*xcart;xp=ap*xp+bp*xcart;
if (theta>thmax);theta=thmax;xp=xss-iccb*xcart;end;
if (theta<-thmax);theta=-thmax;xp=-xss-iccb*xcart;end;
%
% Visualization
%
u=[cos(theta) sin(theta);-sin(theta) cos(theta)];
xpcart=xdcart+xcart;ypcart=htrack+vgap+ydcart;
dbeam=[xdbeam ydbeam]*u';
xpbeam=xcart+dbeam(:,1);ypbeam=htrack+hcart+vgap+vgap+dbeam(:,2);
set(pcart,'Xdata',xpcart,'Ydata',ypcart);
set(pbeam,'Xdata',xpbeam,'Ydata',ypbeam);
drawnow;
%
end
hold off